--- sysdeps/freebsd/cpu.c.orig	2004-07-18 22:33:56.000000000 +0200
+++ sysdeps/freebsd/cpu.c	2005-01-10 09:41:28.000000000 +0100
@@ -79,8 +79,12 @@
 #ifdef KERN_CP_TIME
 	guint64 cpts [CPUSTATES];
 #else
+#ifdef __DragonFly__
+	struct kinfo_cputime cp_time;
+#else
 	long cpts [CPUSTATES];
 #endif
+#endif
 	/* sysctl vars*/
 	struct clockinfo ci;
 	size_t length;
@@ -100,12 +104,19 @@
 		return;
 	}
 #else
+#ifdef __DragonFly__
+	if (kinfo_get_sched_cputime(&cp_time)) {
+		glibtop_warn_io_r (server, "kinfo_get_sched_cputime (cp_time)");
+		return;
+	}
+#else
 	if (kvm_read (server->machine.kd, nlst [0].n_value,
 		      &cpts, sizeof (cpts)) != sizeof (cpts)) {
 		glibtop_warn_io_r (server, "kvm_read (cp_time)");
 		return;
 	}
 #endif
+#endif
 
 	/* Get the clockrate data */
 	length = sizeof (struct clockinfo);
@@ -115,13 +126,29 @@
 	}
 
 	/* set user time */
+#ifdef __DragonFly__
+	buf->user = cp_time.cp_user;
+#else
 	buf->user = cpts [CP_USER];
+#endif
 	/* set nice time */
+#ifdef __DragonFly__
+	buf->nice = cp_time.cp_nice;
+#else
 	buf->nice = cpts [CP_NICE];
+#endif
 	/* set sys time */
+#ifdef __DragonFly__
+	buf->sys = cp_time.cp_sys;
+#else
 	buf->sys = cpts [CP_SYS];
+#endif
 	/* set idle time */
+#ifdef __DragonFly__
+	buf->idle = cp_time.cp_idle;
+#else
 	buf->idle = cpts [CP_IDLE];
+#endif
 
 	/* set frequency */
 	/*
@@ -132,8 +159,13 @@
 	*/
 	buf->frequency = ci.hz;
 	/* set total */
+#ifdef __DragonFly__
+	buf->total = cp_time.cp_user + cp_time.cp_nice
+		+ cp_time.cp_sys + cp_time.cp_idle;
+#else
 	buf->total = cpts [CP_USER] + cpts [CP_NICE]
 		+ cpts [CP_SYS] + cpts [CP_IDLE];
+#endif
 
 	/* Set the flags last. */
 	buf->flags = _glibtop_sysdeps_cpu;
