--- sysdeps/freebsd/mem.c.orig	2004-08-28 22:55:37.000000000 +0200
+++ sysdeps/freebsd/mem.c	2005-01-13 13:12:53.000000000 +0100
@@ -122,8 +122,13 @@
 	struct uvmexp uvmexp;
 	size_t length_uvmexp;
 #else
+#if defined(__DragonFly__)
+        struct vmstats vms;
+        int vms_size = sizeof(struct vmstats);
+#else
 	struct vmmeter vmm;
 #endif
+#endif
 	u_int v_used_count;
 	u_int v_total_count;
 	u_int v_free_count;
@@ -140,12 +145,21 @@
 	 *         value for `vmt.vm'. We use some code from Unix top
 	 *         here.] */
 
+#if defined(__DragonFly__)
+	/* Get the data from sysctlbyname */
+	length_vmt = sizeof (vmt);
+	if (sysctlbyname ("vm.vmtotal", &vmt, &length_vmt, NULL, 0)) {
+		glibtop_warn_io_r (server, "sysctlbyname (vm.vmtotal)");
+		return;
+	}
+#else
 	/* Get the data from sysctl */
 	length_vmt = sizeof (vmt);
 	if (sysctl (mib, 2, &vmt, &length_vmt, NULL, 0)) {
 		glibtop_warn_io_r (server, "sysctl (vmt)");
 		return;
 	}
+#endif
 
 #if defined(__NetBSD__)  && (__NetBSD_Version__ >= 104000000) || defined(__OpenBSD__)
 	length_uvmexp = sizeof (uvmexp);
@@ -154,6 +168,13 @@
 		return;
 	}
 #else
+#if defined(__DragonFly__)
+        /* Get the data by using sysctlbyname */
+        if (sysctlbyname("vm.vmstats", &vms, &vms_size, NULL, 0)) {
+                glibtop_warn_io_r (server, "sysctlbyname (vm.vmstats");
+                return;
+        }
+#else
 	/* Get the data from kvm_* */
 	if (kvm_read (server->machine.kd, nlst[1].n_value,
 		      &vmm, sizeof (vmm)) != sizeof (vmm)) {
@@ -161,6 +182,7 @@
 		return;
 	}
 #endif
+#endif
 
 	if (kvm_read (server->machine.kd, nlst[0].n_value,
 		      &bufspace, sizeof (bufspace)) != sizeof (bufspace)) {
@@ -170,6 +192,9 @@
 
 	/* convert memory stats to Kbytes */
 
+#if defined(__DragonFly__)
+        v_total_count = vms.v_page_count;
+#else
 #if defined(__FreeBSD__)
 	v_total_count = vmm.v_page_count;
 #else
@@ -184,28 +209,42 @@
 		vmm.v_active_count + vmm.v_inactive_count;
 #endif
 #endif
+#endif
 
 #if defined(__NetBSD__)  && (__NetBSD_Version__ >= 104000000) || defined(__OpenBSD__)
 	v_used_count = uvmexp.active + uvmexp.inactive;
 	v_free_count = uvmexp.free;
 #else
+#if defined(__DragonFly__)
+        v_used_count = vms.v_active_count + vms.v_inactive_count;
+        v_free_count = vms.v_free_count;
+#else
 	v_used_count = vmm.v_active_count + vmm.v_inactive_count;
 	v_free_count = vmm.v_free_count;
 #endif
+#endif
 
 	buf->total = (guint64) pagetok (v_total_count) << LOG1024;
 	buf->used  = (guint64) pagetok (v_used_count) << LOG1024;
 	buf->free  = (guint64) pagetok (v_free_count) << LOG1024;
 
+#ifdef __DragonFly__
+        buf->cached = (guint64) pagetok (vms.v_cache_count) << LOG1024;
+#else
 #ifdef __FreeBSD__
 	buf->cached = (guint64) pagetok (vmm.v_cache_count) << LOG1024;
 #endif
+#endif
 
 #if defined(__NetBSD__)  && (__NetBSD_Version__ >= 104000000) || defined(__OpenBSD__)
 	buf->locked = (guint64) pagetok (uvmexp.wired) << LOG1024;
 #else
+#if defined(__DragonFly__)
+        buf->locked = (guint64) pagetok (vms.v_wire_count) << LOG1024;
+#else
 	buf->locked = (guint64) pagetok (vmm.v_wire_count) << LOG1024;
 #endif
+#endif
 
 	buf->shared = (guint64) pagetok (vmt.t_rmshr) << LOG1024;
 
